package com.rinke.solutions.oo.sxw;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import java.util.zip.ZipException;

import javax.xml.transform.stream.StreamSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.output.XMLOutputter;

public class Main {
	
	private static Log log = LogFactory.getLog(Main.class);

	SxwHandler sxw = new SxwHandler();
	private String[] args;

	public Main(String[] args) {
		this.args = args;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Main r = new Main(args);
		r.run();

	}

	private boolean createFlatHtml = false;
	private boolean createHtml = false;
	private String insxw;
	
	String basePath;
	
	private void run() {
		int arg = 0;
		while( arg < args.length ) {
			// parse args
			if( "--createFlatHtml".equals(args[arg]) ) {
				createFlatHtml = true;
				arg++;
			} else if( "--createHtml".equals(args[arg]) ) {
				createHtml = true;
				arg++;
			} else {
				break;
			}
		}
		if( arg == args.length ) {
			usage();
		}
		insxw = args[arg];
		File in = new File(insxw);
		basePath = in.getAbsoluteFile().getParent();
		String name = in.getName();
		try {
			sxw.open( new File(insxw) );
			log.info("creating flat xml ...");
			Document flat = sxw.createFlatXML();
			writeXML(flat, new File( basePath + File.separator + name + "-flat.xml"));
			log.info("creating docbook xml ...");
			Document docbook = sxw.createTransformedDocument(new StreamSource("system/xsl/ooo2sdbk.xsl"));
			writeXML(docbook, new File( basePath + File.separator + name + "-docbook.xml"));
			if( createFlatHtml ) {
				log.info("creating flat html ...");
				File outdir = new File( basePath, "dist/htmlsingle" );
				commonCopy(outdir);
				Document html = sxw.createTransformedDocument(new StreamSource("system/xsl/htmlsingle.xsl"), docbook);
				writeXML(html, new File( outdir, name + "-single.html"));
			}
			if( createHtml ) {
				log.info("creating chunked html ...");
				File outdir = new File( basePath, "dist/html" );
				commonCopy(outdir);
				sxw.createTransformedDocument(new StreamSource("system/xsl/html.xsl"), docbook);
			}
			sxw.close();
		} catch (ZipException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JDOMException e) {
			e.printStackTrace();
		}
	}
	
	private void commonCopy(File outdir) throws IOException {
		outdir.mkdirs();
		copy(false, "system/css/", ".*\\.css", outdir );
		copy(true, "system/images/", ".*", new File(outdir, "/images/" ) );
		copy(false, basePath + "/Pictures/",".*", new File(outdir, "/Pictures/" ) );
	}

	private void copy(boolean rec, String base, String pat, File outdir) throws IOException {
		List<String> files = getFileList(rec,base,pat);
		for (String f : files) {
			copySingleFile(f,outdir, base);
		}
	}

	private void copySingleFile(String filename, File outdir, String base) throws IOException {
		File o = new File(outdir.getAbsolutePath());
		if( filename.startsWith(base)) {
			o = new File( o , new File( filename ).getPath().substring(base.length()) );
		}
		o.getParentFile().mkdirs();
		byte[] buffer = new byte[20000];
		FileInputStream fis = new FileInputStream( filename );
		FileOutputStream fos = new FileOutputStream( o );
		int r = 0;
		while( (r = fis.read(buffer)) > 0 ) {
			fos.write(buffer, 0, r);
		}
		fos.close();
		fis.close();
	}

	private List<String> getFileList(boolean rec, String base, String pat) {
		List<String> res = new ArrayList<String>();
		String[] l = new File(base).list();
		if( l != null  ) {
			for (String aFile : l) {
				boolean isDir = new File(base, aFile).isDirectory();
				if( Pattern.matches(pat,aFile) && !isDir ) {
					res.add(base + aFile);
				}
				if( rec && isDir ) {
					res.addAll(getFileList(rec, base + aFile + "/", pat));
				} 
			}
		}
		return res;
	}

	private void writeXML( Document doc, File out) throws IOException {
		XMLOutputter xmlOut = new XMLOutputter("  ", true, "UTF-8");
		Writer writer = new OutputStreamWriter( new FileOutputStream( out ), "UTF-8" );
		xmlOut.output(doc, writer);
		writer.close();

	}

	private void usage() {
		System.err.println("com.rinke.solutions.oo.sxw.Main <options> SXW-File\n\tOptions: --createHtml --createFlatHtml");
		System.exit(0);
	}

}
