/**
 * SxwHandler.java
 * Created on 28.04.2004 by sr
 *
 * (C) 2003, 2004 by Rinke Solutions, all Rights reserved.
 *
 * Diese Software enthaelt vertrauliche Informationen der Rinke Solutions.
 * Eine Weitergabe dieser Software oder der Informationen als Ganzes oder 
 * in Teilen an Dritte ist nicht gestattet.
 * Die Benutzung darf nur im Rahmen der mit der Rinke Solutions vereinbarten 
 * Bedingungen erfolgen.
 *
 * JEDWEDE GEWAEHRLEISTUNGSANSPRUECHE WEGEN EVENTUELLER MAENGEL AN DER 
 * SOFTWARE SIND, SOWEIT GESETZLICH ZULAESSIG, AUSGESCHLOSSEN. ES WIRD WEDER 
 * GARANTIE NOCH JURISTISCHE VERANTWORTUNG ODER IRGENDEINE HAFTUNG FUER FOLGEN,
 * DIE AUF FEHLERHAFTE ANGABEN, BENUTZUNG ODER FEHLFUNKTIONEN DER SOFTWARE 
 * SELBST ZURUECKGEHEN, UEBERNOMMEN.
 * DER HERSTELLER HAFTET IN KEINEM FALL FUER SCHAEDEN (Z.B. GEWINNVERLUSTE,
 * BETRIEBSUNTERBRECHUNG ODER VERLUSTE VON GESCHAEFTSINFORMATIONEN), 
 * DIE AUFGRUND DER VERWENDUNG ODER DER NICHTANWENDBARKEIT DER SOFTWARE 
 * ODER DER BEGLEITENDEN DOKUMENTATION ENTSTEHEN.
 *
 * $Id: SxwHandler.java,v 1.1 2004/04/29 14:27:44 sr Exp $
 */
package com.rinke.solutions.oo.sxw;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jdom.transform.JDOMResult;
import org.jdom.transform.JDOMSource;

/**
 * @author sr
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class SxwHandler {
	
	ZipFile f;
	
	String basePath;
	
	/**
	 * opens the swx file
	 * @param f File for the swx file
	 * @throws ZipException
	 * @throws IOException
	 */
	public void open( File fr ) throws ZipException, IOException {
		this.f = new ZipFile(fr);
		basePath = fr.getAbsoluteFile().getParent() + File.separator;
		System.setProperty("javax.xml.transform.TransformerFactory", "com.icl.saxon.TransformerFactoryImpl");
	}
	
	/**
	 * gets an entry from the zip file as string
	 * @param pathname what entry
	 * @return the content of that entry as string
	 * @throws IOException 
	 */
	public String get( String pathname ) throws IOException {
		StringBuffer sb = new StringBuffer();
		Reader reader = getReader( pathname);
		if( reader != null ) {
			while(true ) {
				int read = reader.read();
				if( read == -1 ) break;
				sb.append((char)read);
			}
			return sb.toString();
		}
		return null;
	}
	
	public InputStream getStream( String pathname ) throws IOException {
		ZipEntry e = f.getEntry(pathname);
		if( e != null ) {
			return f.getInputStream(e);
		}
		return null;
	}
	
	/** 
	 * gets an Reader for a given pathname
	 * @param pathname
	 * @return the reader
	 * @throws IOException
	 */
	public Reader getReader( String pathname ) throws IOException {
		ZipEntry e = f.getEntry(pathname);
		if( e != null ) {
			// TODO check, if UTF-8 is really always used!!
			String charset = "UTF-8";
			return new InputStreamReader( f.getInputStream(e), charset );
		}
		return null;
	}
	
	Document globalDoc = null;

	/**
	 * creates a flat document as combination of meta and content
	 * for transformation
	 * @return the document
	 * @throws IOException
	 */
	public Document createFlatXML(  ) throws IOException {
		String emptyGlobal = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + 
				"<office:document xmlns:office=\"http://openoffice.org/2000/office\" "+
				"xmlns:style=\"http://openoffice.org/2000/style\" "+
				"xmlns:text=\"http://openoffice.org/2000/text\" "+
				"xmlns:table=\"http://openoffice.org/2000/table\" "+
				"xmlns:draw=\"http://openoffice.org/2000/drawing\" "+
				"xmlns:fo=\"http://www.w3.org/1999/XSL/Format\" "+
				"xmlns:xlink=\"http://www.w3.org/1999/xlink\" "+
				"xmlns:number=\"http://openoffice.org/2000/datastyle\" "+
				"xmlns:svg=\"http://www.w3.org/2000/svg\" "+
				"xmlns:chart=\"http://openoffice.org/2000/chart\" "+
				"xmlns:dr3d=\"http://openoffice.org/2000/dr3d\" "+
				"xmlns:math=\"http://www.w3.org/1998/Math/MathML\" "+
				"xmlns:form=\"http://openoffice.org/2000/form\" "+
				"xmlns:script=\"http://openoffice.org/2000/script\" "+
				"xmlns:dc=\"http://purl.org/dc/elements/1.1/\" "+
				"xmlns:meta=\"http://openoffice.org/2000/meta\" "+
				"office:class=\"text\" office:version=\"1.0\">" + 
				"</office:document>";
		
		URL systemId = new URL("file:///"+basePath + "system/oo-dtd/test");
		
		if( globalDoc == null ) {
			
			SAXBuilder builder = new SAXBuilder(false);
			try {
				globalDoc = builder.build(new StringReader(emptyGlobal));
				Element globalRoot = globalDoc.getRootElement();
				
				List metaChilds = builder.build( getReader( "meta.xml"), systemId.toExternalForm() ).getRootElement().getChildren();
				List contentChilds = builder.build( getReader( "content.xml"), systemId.toExternalForm() ).getRootElement().getChildren();
				
				Iterator i = metaChilds.iterator();
				while (i.hasNext()) {
					globalRoot.addContent( (Element)((Element)i.next()).clone() );
				}
				i = contentChilds.iterator();
				while (i.hasNext()) {
					globalRoot.addContent( (Element)((Element)i.next()).clone() );
				}
				
				transformAndExtractImages( globalRoot );
				
			} catch (JDOMException e) {
				e.printStackTrace();
			}
			
		}
		return globalDoc;
	}

	Namespace imageNs = Namespace.getNamespace("draw", "http://openoffice.org/2000/drawing");
	Namespace xlinkNs = Namespace.getNamespace("xlink", "http://www.w3.org/1999/xlink");

	@SuppressWarnings("unchecked")
	private void transformAndExtractImages(Element e) {
		List<Element> images = e.getChildren("image", imageNs);
		for (Element img : images) {
			String link = img.getAttributeValue("href", xlinkNs);
			if( link.startsWith("#")) {
				link = link.substring(1);
				img.setAttribute("href", link, xlinkNs);
				dumpImage(link);
			}
			//System.out.println(link);
		}
		List<Element> c = e.getChildren();
		for (Element child : c) {
			transformAndExtractImages(child);
		}
	}

	private void dumpImage(String link) {
		try {
			int buflen = 30000;
			byte[] buffer = new byte[buflen];
			int r = 0;
			InputStream is = getStream(link);
			new File(basePath + link).getParentFile().mkdirs();
			FileOutputStream os = new FileOutputStream(basePath + link);
			while( (r = is.read(buffer)) > 0 ) {
				os.write(buffer,0,r);
			}
			os.close();
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * creates a transformed Document
	 * @param stylesheet the transforming stylesheet (its stream source)
	 * @return the tranformed document
	 * @throws IOException
	 * @throws JDOMException
	 */
	public Document createTransformedDocument(StreamSource stylesheet, Document doc) throws IOException, JDOMException {
		try {
			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer(stylesheet);
			JDOMResult out = new JDOMResult();
			transformer.transform(new JDOMSource(doc), out);
			return out.getDocument();
		} catch (TransformerException e) {
			throw new JDOMException("XSLT Transformation failed", e);
		}
		
	}
	/**
	 * creates a transformed Document
	 * @param stylesheet the transforming stylesheet (its stream source)
	 * @return the tranformed document
	 * @throws IOException
	 * @throws JDOMException
	 */
	public Document createTransformedDocument(StreamSource stylesheet) throws IOException, JDOMException {
		return createTransformedDocument(stylesheet, createFlatXML() );
	}
	/**
	 * creates a transformed Document
	 * @param stylesheet the transforming stylesheet (its stream source)
	 * @return the tranformed document
	 * @throws IOException
	 * @throws JDOMException
	 */
	public Document createSingleHtml(StreamSource toDocbook, StreamSource toTarget) throws IOException, JDOMException {
		Document doc = createFlatXML();
		try {
			Transformer transformer = TransformerFactory.newInstance()
					.newTransformer(toDocbook);
			JDOMResult out = new JDOMResult();
			transformer.transform(new JDOMSource(doc), out);
			
			transformer = TransformerFactory.newInstance().newTransformer(toTarget);
			JDOMResult out2 = new JDOMResult();
			transformer.transform(new JDOMSource(out.getDocument()), out2);
			return out2.getDocument();
		} catch (TransformerException e) {
			throw new JDOMException("XSLT Transformation failed", e);
		}

	}

	
	/**
	 * closes the swx file
	 * 
	 * @throws IOException
	 */
	public void close(  ) throws IOException {
		f.close();
	}
}
